function [ K , flag] = Krawczyk( X )
%KRAWCZYK   -- This is the K'(X,\bar{x}) -- 
%           -- See Definition 3.2 --

%           -- if flag ==3, then the hypothesis of Theorem 3.4 holds
%              that is, K \subset X
%               else flag=0.


% (A)   We fix a point x_hat (non-interval).
% (B)   We calculate the 2Mx2M matricies A and A^dagger.
% (C)   We fix a point x_hat (interval).
% (D)   We calculate F_M(x_hat).
% (E)   We calculate DF(X)
% (F)   We calculate X - x_hat
% (G)   We calculate g_M and g_Infty
% (H)   We calculate K'_M   --Line (47)
% (I)   We calculate K'_infty and K'.

%   Afterwards we calculate Step 6 in Prune.

%---------------------
%(A)   We fix a point x_hat (non-interval).
    
    % We define x_hat as a [NON-INTERVAL] complex vector in the center of X.
    a_Hat = mid(X.A);
    b_Hat = mid(X.B);
    omega_Hat = mid(X.omega);
    % We make a cube
    X_hat_double = Cube(X.alpha,omega_Hat,a_Hat,b_Hat,X.M);
    
    % We make X_hat a double so it is faster to computer
    X_hat_double.Double

    
%(B)   We calculate the 2Mx2M matricies A and A^dagger.
    [ dw_R, dw_I, da_R , da_I, db_R, db_I ]=DFeval( X_hat_double );
    
    % We put omega in the imaginary part of c_1.
    db_R(:,1) = dw_R;
    db_I(:,1) = dw_I;
    A_mat = [ da_R     , db_R     ; da_I   , db_I];

    % We define A.
    A_dagger = A_mat^-1;
    
    A_mat   = intval(A_mat);
    A_dagger = intval(A_dagger);
    
    
%(C)   We fix a point x_hat (interval).
    a_Hat_int       = intval(a_Hat);
    b_Hat_int       = intval(b_Hat);
    omega_Hat_int   = intval(omega_Hat);
    

    
    % We define X_hat_interval as the thin-interval cube of x_hat
    X_hat_interval = Cube(X.alpha,omega_Hat_int,a_Hat_int,b_Hat_int,X.M);
    


%(D)   We evaluate F_M(x_hat)
    [ out_R, out_I ] = Feval(X_hat_interval);

    F_hat_RI = [ out_R ; out_I];
     
    
   
%(E)    We calculate  DF(X) 
    [ dw_R, dw_I, da_R , da_I, db_R, db_I ] = DFeval(X );
    
    % We put omega in the imaginary part of c_1.
    db_R(:,1) = dw_R;
    db_I(:,1) = dw_I;
    DF = [ da_R     , db_R     ; da_I   , db_I];

%(F)    We calculate X - x_hat [INTERVAL]
    recentered_omega = X.omega  - omega_Hat_int;
    recentered_a     = X.A      - a_Hat_int;
    recentered_b     = X.B      - b_Hat_int;


    rc_C_R      = recentered_a ;
    rc_C_I      = recentered_b ;
    rc_C_I(1)   = recentered_omega;
    rc_C_RI     = [rc_C_R ;rc_C_I ];

    
%(G) We calculate gM 
    g_M = gM( X );
    % As an interval, this makes it contain all vectors v such that 
    % |v|_k < |gM|_k

    % We make this sum it positive/negative
    g_M = infsup(-1,1)*g_M;
    % We calculate gInfty
    g_infty = gInfty( X );
    

%(H)   We calculate K'_M   -- Line (47)
%          K'_M := x_hat - A^dag.F(x_hat) + ( I - A^dagger*A ).(X-x)
%                 + ( I - A.DF(X) ).(X-x)  +/-  A^dagger g_M(X)

    % We define c_hat as an [INTERVAL] real vector.
    C_hat_R     = a_Hat_int;
    C_hat_I     = b_Hat_int;
    C_hat_I(1)  = omega_Hat_int;
    C_hat_RI    = [C_hat_R ;C_hat_I];


    % We calculate the last summand of the first line of (47) 
    %  ( I - A^dagger*A ).(X-x)
    defect_inverse = (eye(2*X.M) - A_dagger*A_mat) * rc_C_RI;
    
    
    
    % We calculate the first line of (47)
    %       x_hat - A^dag.F(x_hat) + ( I - A^dagger*A ).(X-x)
    KM1 = C_hat_RI-  A_dagger*F_hat_RI + defect_inverse;
    % We calculate the second line of (47)
    %       + ( I - A.DF(X) ).(X-x)  +/-  A^dagger g_M(X)
    KM2 = ((eye(2*X.M) - A_dagger* DF)*rc_C_RI)   + (A_dagger * g_M);

    % We calculate (47).    
    KM = KM1 + KM2 ;

%(I)   We calculate K'_infty and K'.    
    
    % We produce the cube K
    K       = Cube;
    K.alpha = X.alpha;
    K.s     = X.s;
    K.C0    = g_infty;
    
    K.omega = KM(X.M+1) ;    
    K.A 	= KM(1:X.M); 
    K.B     = KM(X.M+1:2*X.M);
    K.B(1)  = intval(0);
%------------------------------------------    
    
    
   % 6. We check to see if K' is strictly inside X.
   
   % 6a We check for strict tail inclusion.
   %    i.e. g_\infty(X) < C_0
   if  (sup(K.C0) < sup(X.C0))
       % We create a 1x2M vector for X
       X_vec        = [ X.A;X.B];
       X_vec(X.M+1) = X.omega;
        % 6b We check if K'_M \subseteq X_M
        if ~any(~in(KM,X_vec))
        % The 'in' function will produce 1 if there is inclusion
            
            % 6c We check for:  K' + A^dag*F(x) [= int(X)
            % KM_adj :=  K'_M + A^dag*F(x)        
            KM_adj = KM + A_dagger*F_hat_RI;
            
            % we check to see if sup(KM_adj) < sup(X_vec)           
            flag_sup = ~any(~(sup(KM_adj)<sup(X_vec)));
            % we check to see if inf(KM_adj) > inf(X)
            flag_inf = ~any(~(inf(KM_adj)>inf(X_vec)));
            
            if flag_sup  && flag_inf
                disp('Unique Zero')
                flag = 3;
                return
            end
        end
    end
    
    
    flag = 0;
   
end

