function [ g_infty ] = gInfty( X )
%gInfty  -- This defines  the function  g_\infty 
%        -- See Definition 3.2        --
%        -- See also Definition 2.11  --
%   g_infty - Output is a real number (interval).

    % 0. We make useful variable definitions
    
     % These weights are for k^s from M+1 <=k<=2M
     % Used in Eq (24) & Eq (25).
    weights_Mp1_2M =  transpose(  intval(X.M+1:2*X.M)).^(X.s) ;
    
    % We define x_bar as a complex vector
    a_Bar       = intval(mid(X.A));
    b_Bar       = intval(mid(X.B));
    alpha_Bar   = intval(mid(X.alpha));
    omega_Bar   = intval(mid(X.omega));

    % We define X - \bar{x}
    recentered_omega = X.omega - omega_Bar;
    recentered_a = X.A - a_Bar;
    recentered_b = X.B - b_Bar;
    
    % We create a column vector of |H|_j, |X|_j, |\bar{x}|_j 
    H_M     = sqrt(recentered_a.^2  + recentered_b.^2);
    C       = sqrt((X.A).^2         + (X.B).^2);
    C_bar   = sqrt((a_Bar).^2       + (b_Bar).^2);    
    
    %%%%%%%%%%%%%%%%%%%%
    % Definition 2.11  %
    %%%  g_infty^i   %%%
    %%%%%%%%%%%%%%%%%%%%

    %%% We calculate 
    %                          M
    %       max        k^s  SUM       |cbar|_j  |cbar|_k-j
    % M+1 <= k <= 2 M         j=k-M   
       
    % We compute the sum over all k as a matrix-vector product
    % MATRIX = |cbar|_k-j   ;; VECTOR = |cbar|_j     
    % We initialize the coefficient matrix.
    decay_M1 = X.Zeros;    
    
    % Let k_prime = k'.
    % Let k = M + k' , and 1 <= k' <= M .
    
    % We want the k' row of our coefficient matrix 
        % to equal '0'   (non-zeros start at k')
        %       for its entries between             1  --- k' -1 
        % to range from |cbar|_M down to |cbar|_k' 
        %       for the entries between             k' --- M 
    
    % We flip it to reverse it for |cbar|_k-j.
    Row = fliplr(transpose(C_bar));  
    % We count up to M.
    for k_prime = 1:X.M
        % We put 'Row' into the k'^th row.
        decay_M1(k_prime,:) =Row;
        % We shift 'Row' to the right. 
         Row = [0,  Row(1:end-1)];
        % We increase to the next row. 
    end    
        
    % We calculate the sum part as a matrix-vector product
    g_infty_i = decay_M1*C_bar;
    % We scale with k^s from M+1 to 2M
    g_infty_i = weights_Mp1_2M .* g_infty_i;
    % We calculate the maximum. 
    g_infty_i = intval(max(g_infty_i.sup));

    
    %%%%%%%%%%%%%%%%%%%%%
    %%%% g_infty_ii   %%%
    %%%%%%%%%%%%%%%%%%%%%
    
    %%%%%%%%%%%%%%%%%%%%
    % Definition 2.11  %
    %%% g_infty^ii,a %%%
    %%%%%%%%%%%%%%%%%%%%
    
    
    
    % Finite-Finite    
        % This calculation is very similar to the one for g_infty^i

        %                            M
        %       max        k^s  SUM       |H|_j  |X|_k-j
        % M+1 <= k <= 2 M           j=k-M   

        % We compute the sum over all k as a matrix-vector product
        % MATRIX = |X|_k-j   ;; VECTOR = |H|_j     
        % We initialize the coefficient matrix.        
        finfin = X.Zeros;    
        
        % Let k_prime = k'.
        % Let k = M + k' , and 1 <= k' <= M .

        % We want the k' row of our coefficient matrix 
        % to equal '0'   (non-zeros start at k')
        %       for its entries between             1  --- k' -1 
        % to range from |X|_M down to |X|_k' 
        %       for the entries between             k' --- M 
        
        % We flip it to reverse it for |X|_k-j.
        Row_ii  = fliplr(transpose(C));  
        % We count up to M.
        for k_prime = 1:X.M
            % We put 'row' into the k^th row.
            finfin(k_prime,:) =Row_ii;
            % We shift 'row' to the right. 
             Row_ii = [0,  Row_ii(1:end-1)];
        end    

        % We scale with k^s for M+1 <=k<=2M
        little_sum_1 = weights_Mp1_2M .*(finfin*H_M);
        % We take the maximum
        little_sum_1 = intval(max(little_sum_1.sup));
    
    % Finite-Infinite
        % This is just vector <dot product> vector
        % We make the coefficients
        %             (  M+j+1   )^s
        %             ( -------- )    + 1
        %             (    M+1   )
    
        weights_3 = intval(1:X.M)+1+X.M;
        weights_3 = weights_3 ./(X.M+1);
        weights_3 = weights_3 .^(X.s) +1;
        % The Dot Product
        little_sum_2 = X.C0*weights_3*(C+H_M);

    % Infinite-Infinite
        little_sum_3 = 2*X.C0^2*(2^(X.s)+1) / ((X.s-1)*X.M^(X.s-1));

        
    g_infty_iia =(little_sum_1+little_sum_2+little_sum_3);

    
    
    %%%%%%%%%%%%%%%%%%%%
    % Definition 2.11  %
    %   g_infty^ii,b   %
    %%%%%%%%%%%%%%%%%%%%
    
    % This defines the numbers k^s used in calculating the \Omega^s norm
    weights = intval(transpose(1:X.M)).^(X.s);
    
    % We calculate C1 = sup{x\in X} ||x||_s          
        C1 = C.*weights;
        C1 = max(C1.sup);
        C1 = intval(max( [C1, sup(X.C0)]));
    
    
    % We calculate gamma_{M+1}
    gamma = (4* log(intval(X.M-1))/intval(X.M+1)  +(intval('pi')^2-6)/intval(3)) * (2/intval(X.M+1) +1/intval(2))^intval(X.s-2);
    gamma = 2*((X.M+1)/(intval(X.M)))^intval(X.s) + gamma;

    
    g_infty_iib = X.C0*C1 *(  (X.s-1)/(  (X.M+2) *(X.s-2) )  + X.s/(X.s-1) );
    g_infty_iib = g_infty_iib  + C1^2*gamma/2;
    
        
    %%%%%%%%%%%%%%%%%%%%
    %  Definition 3.2  %
    %  g_infty^ii,"c"  %
    %%%%%%%%%%%%%%%%%%%%
    % We calculate the summand in g_infty^ii which is not multiplied by 
    %  either    g_infty^ii,a   nor  g_infty^ii,b
    

    g_infty_iic = abs(1- (alpha_Bar/X.alpha) * (X.omega/omega_Bar));
    g_infty_iic = g_infty_iic + (alpha_Bar/(omega_Bar*(X.M+1)) );
    g_infty_iic = g_infty_iic *X.C0;
    
    
    
    %%%%%%%%%%%%%%%%%%%%
    %  Definition 3.2  %
    %    g_infty^ii    %
    %%%%%%%%%%%%%%%%%%%%
    

    % We define the three parts of g_infty_ii
    Summand_ii_A = 2 * alpha_Bar /( (X.M+1)* omega_Bar)* g_infty_iia ;
    Summand_ii_B = abs(recentered_omega)*(alpha_Bar/omega_Bar)* ( (1+1/X.alpha)*X.C0 +g_infty_iib) ;
    Summand_ii_C = g_infty_iic;
        
    g_infty_ii = Summand_ii_A + Summand_ii_B + Summand_ii_C ;
    
    %%%%%%%%%%%%%%%%
    
    %%%%%%%%%%%%%%%%%%%%
    %  Definition 3.2  %
    %     g_infty      %
    %%%%%%%%%%%%%%%%%%%%
    
    g_infty = ( alpha_Bar /( omega_Bar*(X.M+1) ))*g_infty_i  +   g_infty_ii;
 
    % We take a supremum here at the end
    g_infty = intval(sup(g_infty));


end

