function [ List_S , cubes_left_over ] = InitialPrep(tmin, res, alpha, List_Q, List_Qbar, List_X_I, M,subdivisions)
%InitialPrep    -- This takes the bounds from [JLM] and creates Fourier bounds
%               -- See Algorithm 5.7 --
%  The output is "List_S", the formal output of Algorithm 5.7, and 
%   "cubes_left_over", which is the numberr of cubes in S''', after Step 7
%
% The inputs are given as follows 
%     alpha     -   The parameter value for Wright's equation
%     M         -   The (complex) Galerkin projection dimension
%     S         -   While part of Algorithm 5.7, this is NOT an input to
%                   "InitialPrep". The variable S is hard-coded to be 3.
% subdivisions  -   This is refered to as << N >> in Algorithm 5.7
%
% These inputs are retrieved as part of Step 1 of Algorithm 5.7
%     List_Q    -   List of interval bounds on the amount of time the sops is << POSITIVE >>
%     List_Qbar -   List of interval bounds on the amount of time the sops is << NEGATIVE >>
%     List_X    -   List of Interval bounding functions for the exponential
%                    verions of Wright's equation
%
%  These are some computational paramters needed for the bounding functions
%     tmin -    The min
%     res  -    The time resolution of the bounding functions
%
% See also the paper ``Stability and uniqueness of slowly oscillating periodic solutions to Wrights equation''
%   for further discussion how the bounding functions are stored.  



%%% 1. We receive the Integration bounds as input.

% 2&3. We use the change of variables to convert the SOPS 
%           from the exponential form to the quadratic form.
%      This then define bounding functions 
%           for Y^s, with  0 <= s <= S.         (S=3)
%      This is done for all the functions in "List_X_I" at once
    [ List_Y_I, List_DY_I ,List_DDY_I, List_DDDY_I ] = X2Y( res, alpha, List_X_I);




    
    
     % We iterate through all bounding functions in << List_X_I >>
     % Step 4-7 are performed inside this for loop
     %   << List_X  >>, a collection of cubes initialized below,
     %                 is \mathcal{S}''' in Algorithm 5.7 
     List_X ={};
    
     for k = 1:length(List_Y_I)
     k
        % 4. We calculate the Fourier projection 
        Y    = List_Y_I{k};
        DY   = List_DY_I{k};
        DDY  = List_DDY_I{k};
        DDDY = List_DDDY_I{k};   
        Q = List_Q{k};
        Qbar = List_Qbar{k};

        [flag, X ] = FourierProjection(tmin , res, Q, Qbar, M , Y,DY,DDY,DDDY);
        % If an empty intersection is produced in "Fourier Projection" 
        %   then the resulting cube cannot contain any solutions 
        if flag == 1
            continue
        end
        
        % 5. We Impose our phase condition
        PhaseCondition( X );
        
        % 6. We add \alpha to the cubes.
        X.alpha = alpha;

        % 7. We Prune X, removing it if necessary.
        [ flag] = Prune( X );
        
        if flag == 1
            continue
        else
            List_X{end+1}=X;
        end

    end% iterating through List_Y
     
    disp('Cubes Left over')
    cubes_left_over = length(List_X)
    
    %%%-----------------------------------------------------
    %%%-----------------------------------------------------

    
    
    % 8a. We subdivide the omega x a_1 space covered by List_X into an N x N grid 
    
    % We define the interval I^\omega 
        Wmin = List_X{1}.omega.inf;
        Wmax = List_X{1}.omega.sup;
        for k = 2:length(List_X)
            omega = List_X{k}.omega;

            if omega.sup > Wmax
                Wmax = omega.sup;
            end
            if omega.inf < Wmin
                Wmin = omega.inf;
            end
        end
        % We define I^\omega 
        W = infsup(Wmin,Wmax);
    
    
    % We define the interval I^{a_1}
        A1min = inf(List_X{1}.A(1));
        A1max = sup(List_X{1}.A(1));
        for k = 2:length(List_X)
            A1 = List_X{k}.A(1);

            if A1.sup > A1max
                A1max = A1.sup;
            end
            if A1.inf < A1min
                A1min = A1.inf;
            end
        end
        % We define I^{a_1}
        A1 = infsup(A1min,A1max);
      
    % 8b. We divide I^a1 and I^omega into an NxN grid iteratively, within two <<for>> loops.    
        % We calculate the width of I^{a_1} & I^{omega} when divided into N-subdivisions
        delta_A1 = intval(2*rad(A1)) / subdivisions;
        delta_W  = intval(2*rad(W )) / subdivisions;

        % We create the initial (bottom left) intervals for omega and a1.
        Iw = infsup(Wmin, sup(delta_W+Wmin));
        Ia1= infsup(A1min,sup(delta_A1+A1min));
        Ia1_initial=Ia1;
    
        
% We initialize the output.
    List_S ={};   

    % 9. We iterate through our {omega, a1} grid, looking at the grid element:
    %       { Iw , Ia1 }.
    %    We then find all cubes in List_X whose omega and a1 projections
    %       intersect Iw and Ia1.
    %    If there is an intersection, we take a convex hull of this cube
    %       with all other cubes intersecting the grid element. 
    %    The omega and a1 values should be no bigger than Iw and Ia1.
    
    % We iterate through all the Iw intervals once
    for k_w = 1:subdivisions
        % For a new Iw interval, we start iterating through Ia1 starting at
        % the beginning
        Ia1=Ia1_initial;
        for k_a = 1:subdivisions
            % We check for cubes which intersect { Iw,  Ia }
            
            % Raise a flag here, starting at value 1, to see if anything 
            %   intersects the grid element. 
            flag_grid =1;
            % If something does intersect for the first time, 
            %   then we make a new instance of a cube. Then set flag :=0
            % If something intersects a secondtime, we modify the cube we
            %   created.
            % If the flag was never lowered, then we do not need to add a
            %   result to the final output.
            
            % We iterate through all cubes in List_X
            for k = 1:length(List_X)
                % Check to see if things intersect
                [empty_W,  Iw_local  ] = emptyintersect(Iw ,List_X{k}.omega);
                [empty_A1, Ia1_local ] = emptyintersect(Ia1,List_X{k}.A(1));
                
                if (~empty_W)&& (~empty_A1)
                    % There is a non-empty intersections
                    if flag_grid ==1
                        % this is the first non-empty intersection
                        X_beta = copy(List_X{k});
                        X_beta.A(1) = Ia1_local;
                        X_beta.omega = Iw_local;
                        flag_grid =0;
                    else
                        % This is NOT the first non-empty intersection
                        
                        % We take the hull this new cube, 
                        %   with the hulls of all the previous cubes
                        new_a1 = hull(X_beta.A(1),Ia1_local);
                        new_w = hull(X_beta.omega,Iw_local);
                        
                        X_beta.A = hull(X_beta.A,List_X{k}.A);
                        X_beta.B = hull(X_beta.B,List_X{k}.B);
                        X_beta.C0 = hull(X_beta.C0,List_X{k}.C0);
                        
                        X_beta.A(1) = new_a1;
                        X_beta.omega = new_w;
                    end
                end% What to do if a cube intersects the grid element 
            end% Cubes in List_X
            
            if flag_grid == 0 
                % In this case, at least one cube in List_X 
                %  had a non-empty intersection with the given grid element
                % We add X_beta to our output
                List_S{end+1} = X_beta;    
            end
            
            % We increase the interval of a1
            % note: delta_A1 is an intval-interval
            Ia1 = Ia1 + delta_A1;
        end% Subdivisions of a1
        
        % We increase the interval of omega
        % note: delta_W is an intval-interval
        Iw = Iw + delta_W;
        
    end% Subdivisions of omega 
    
end%-InitialPrep

